// MenuKey
// © 2000 Sum Software
// Sander R.M. Stoks - sanders@sumware.demon.nl
// This code is in the public domain.

#include "MenuKey.h"
#include <Beep.h>
#include <InterfaceDefs.h>
#include <View.h>	// for some definitions
#include <List.h>
#include <Application.h>

#define DEVICE_NAME "Menu Key Device"

thread_id	  MenuKey::fThread = B_ERROR;
bool		  MenuKey::fActive = false;
bool		  MenuKey::fValid = false;

BInputServerDevice *instantiate_input_device()
{
	return (new MenuKey());
}

MenuKey::MenuKey ()
{
	input_device_ref keyDevice = { DEVICE_NAME, B_POINTING_DEVICE, NULL };
	input_device_ref *keyDeviceList[2] = { &keyDevice, NULL };
	
	if (RegisterDevices (keyDeviceList) != B_OK)
		fValid = false;
	else
		fValid = true;
}

MenuKey::~MenuKey ()
{
}

status_t MenuKey::InitCheck()
{
	return BInputServerDevice::InitCheck();
}

status_t MenuKey::SystemShuttingDown()
{
	return BInputServerDevice::SystemShuttingDown();
}

status_t MenuKey::Start (const char *device, void *cookie)
{
	if (fValid)
	{
		fActive = true;
		fThread = spawn_thread (key_thread, device, B_LOW_PRIORITY, this);
		resume_thread (fThread);
		return B_OK;
	}
	else
		return B_ERROR;
}

status_t MenuKey::Stop (const char *device, void *cookie)
{
	if (fValid)
	{
		status_t err = B_OK;
		fActive = false;
		wait_for_thread (fThread, &err);
		fThread = B_ERROR;
		return B_OK;
	}
	else
	{
		fActive = false;
		return B_OK;
	}
}

status_t MenuKey::Control (const char *device, void *cookie, uint32 code, BMessage *msg)
{
	if (strcmp (device, DEVICE_NAME))
		return B_OK;	// It's not for us...

	// do something?

	return B_OK;
}

int32 MenuKey::key_thread (void *arg)
{
	MenuKey *me = (MenuKey *) arg;
	
	while (me->fActive && me->fValid)
	{
		key_info key;
		get_key_info (&key);
		if (key.modifiers & B_MENU_KEY)
		{
#if DO_RAISE
			BMessage raise (B_COUNT_PROPERTIES);
			BMessage reply;

			// Find the Deskbar
			BMessenger Deskbar;
			BList team_list;
			team_id teamid;
			app_info appinfo;
			
			be_roster->GetAppList (&team_list);
			for (int32 i = 0; i < team_list.CountItems(); i++)
			{
				teamid = (team_id) team_list.ItemAt (i);
				be_roster->GetRunningAppInfo (teamid, &appinfo);
				if (!strcmp (appinfo.ref.name, "Deskbar"))
				{
					Deskbar = BMessenger (0, teamid);
					break;
				}
			}

			Deskbar.SendMessage (&raise, &reply);
			// I'm using a BeOS bug here which raises an app's window
			// when it gets sent a B_COUNT_PROPERTIES message.
			// As soon as Be fixes this, I'm toast.
			// (they did fix it in Maui, so I commented it out.
			// If anyone knows of a way to raise windows not belonging
			// to your own app (via scripting?) please let me know!)
#endif
			BRect deskframe;
			get_deskbar_frame (&deskframe);
			float x, y;
			if (deskframe.left == 0)
				x = 0;
			else
				x = (deskframe.left + 3)/deskframe.right;
			
			if (deskframe.top == 0)
				y = 0;
			else
				y = (deskframe.top + 3)/deskframe.bottom;
			
			// I would have loved to save the mouse position here
			// (i.e. simulate the click, and move the mouse back
			// to where it was).
			// There is a set_mouse_position() in the Game Kit,
			// but there is no get_mouse_potition().  I tried it
			// with using a BView in here and using it's GetMouse(),
			// but that kept crashing.  I'd be interested if anyone
			// can think of a solution!
			
			BMessage *event = new BMessage (B_MOUSE_MOVED);
			event->AddInt64 ("when", system_time());
			event->AddFloat ("x", x);
			event->AddFloat ("y", y);
			me->EnqueueMessage (event);

			event = new BMessage (B_MOUSE_DOWN);
			event->AddInt64 ("when", system_time());
			event->AddInt32 ("buttons", B_PRIMARY_MOUSE_BUTTON);
			event->AddFloat ("x", x);
			event->AddFloat ("y", y);
			me->EnqueueMessage (event);

			event = new BMessage (B_MOUSE_UP);
			event->AddInt64 ("when", system_time());
			event->AddFloat ("x", x);
			event->AddFloat ("y", y);
			me->EnqueueMessage (event);
		}
		snooze (50000);
	}
	return B_OK;
}